@@ -1,8 +1,11 @@
 from copy import deepcopy
+from datetime import datetime
 from math import ceil
 from re import match
+from time import sleep
 
-from autoconv.telegram_data import TelegramData
+from autoconv.utils.telegram_data import TelegramData
+from autoconv.utils.timed_thread import TimedThread
 from telegram import InlineKeyboardButton, InlineKeyboardMarkup
 from telegram.error import BadRequest
 from telegram.ext import ConversationHandler
@@ -12,15 +15,16 @@ class AutoConvHandler:
     def __init__(self, conversation, telegram_state_name):
         self.conversation = conversation
         self.NEXT = telegram_state_name
-        self.tData = TelegramData(conversation.users_list)
+        self.tData = TelegramData(self, conversation.users_list)
         self.prev_state = None
         self.curr_state = conversation.start
         self._bkup_routes, self._list_keyboard, self._bkup_indexes = None, None, dict()
         self.list_labels = None
         self.conversation._check_routes()
         self.conversation._set_states_text()
+        self.threads = list()
 
-    # ---- Next state
+    # -------- changing state --------
 
     def _build_keyboard(self, state):
         """Build Keyboard for callback state"""
@@ -83,28 +87,36 @@ class AutoConvHandler:
             {"prev_state": self.prev_state.name, "state": new_state.name}
         )
         if new_state.regex or new_state.handler:
-            data_context.update({"error": False})
+            data_context.update({"error": False, "error-counter": 0})
         elif data_context.get("error") is not None:
             data_context.pop("error")
+            data_context.pop("error-counter")
         self._restore_basic_routes()
         return new_state
 
-    def _wrong_message(self, text=True):
+    def _wrong_message(self, text, data):
         """Handler for wrong message"""
         self.tData.update.message.delete()
         state = self.conversation.get_state(self.tData.context.user_data.get("state"))
         self._restore_basic_routes()
+        # save data
+        error_counter = self.tData.context.user_data.get("error-counter", 0)
+        error_values = {"error": True, "error-counter": error_counter + 1}
+        self.tData.context.user_data.update(error_values)
+        self.tData.context.user_data.get("data").update({state.name: data})
+        # update msg
         error_text = text and state.regex_error_text or state.handler_error_text
-        if error_text and not self.tData.context.user_data.get("error"):
-            self.tData.context.user_data.update({"error": True})
-            self._send_message(
-                state,
-                self.tData.context.user_data.get("bot-msg").edit_text,
-                post_func=lambda x: f"{x}\n\n{error_text}",
-            )
+        func = (
+            lambda x: x + f"\n\n{error_text}"
+            if error_text and not state.dynamic_text
+            else x
+        )
+        self._send_message(
+            state, self.tData.context.user_data.get("bot-msg").edit_text, post_func=func
+        )
         return self.NEXT
 
-    # ---- Dynamic stuff
+    # -------- Dynamic stuff --------
 
     def _update_dynamic_list(self, state):
         """Update i and routes backup for dynamic list"""
@@ -193,9 +205,17 @@ class AutoConvHandler:
         self.conversation.users_list = new_users
         self.tData.users = new_users
 
+    def _build_dynamic_text(self, state):
+        regex, message = state.dynamic_text(self.tData.prepare())
+        state.add_text(regex is None and r"^.*$" or regex, message)
+        if self.tData.udata.get("error") and state.regex_error_text:
+            return f"\n\n{state.regex_error_text}"
+        return ""
+
     def _build_dynamic_stuff(self, state):
-        """Compute dynamic stuff: action > routes > keyboard > list > refresh"""
+        """Compute dynamic stuff: action > text > routes > keyboard > list > refresh"""
         data = self.tData.context.user_data
+        msg = state.msg
         if self.prev_state != self.curr_state and data.get("list"):
             data.pop("list")
             last_i = data.pop("list_i")
@@ -206,6 +226,8 @@ class AutoConvHandler:
             self._update_dynamic_list(state)
         if state.action:
             action_str = state.action(self.tData.prepare())
+        if state.dynamic_text:
+            msg += self._build_dynamic_text(state)
         if state.routes:
             self._build_dynamic_routes(state)
         if state.build:
@@ -215,10 +237,10 @@ class AutoConvHandler:
             keyboard = self._build_dynamic_list(state, keyboard)
         if state.refresh_auth:
             self._refresh_auth_users(state)
-        reply_msg = state.msg.replace("@@@", state.action and action_str or "")
+        reply_msg = msg.replace("@@@", state.action and str(action_str) or "")
         return InlineKeyboardMarkup(keyboard), reply_msg
 
-    # ---- Dev functions
+    # -------- Dev functions --------
 
     def _restore_basic_routes(self):
         if not self._bkup_routes:
@@ -235,7 +257,7 @@ class AutoConvHandler:
         """Initializate user data in context"""
         if not self.tData.context.user_data:
             self.tData.context.user_data.update(
-                {"prev_state": None, "state": state.name, "error": False, "data": {}}
+                {"prev_state": None, "state": state.name, "data": {}}
             )
 
     def _handle_bad_request(self, exception, update, state):
@@ -260,6 +282,7 @@ class AutoConvHandler:
             reply_markup=keyboard,
             **(self.conversation.defaults | state.kwargs),
         )
+        self.tData.context.user_data.update({"last-message-time": str(datetime.now())})
         if save:
             self.tData.context.user_data.update({"bot-msg": new_msg})
         return new_msg
@@ -270,8 +293,53 @@ class AutoConvHandler:
             return True
         return False
 
-    # ---- Public functions
+    # -------- Public functions ---------
 
+    # ---- async
+    def set_timed_function(self, seconds, state=None, function=None):
+        """Start a timer to change the state"""
+        self.stop_timed_function()
+
+        def _timed_force_state():
+            if function:
+                function(self.tData.prepare())
+            if state:
+                self.force_state(state, self.tData.update)
+
+        name = state and (isinstance(state, str) and state or state.name)
+        name = name or function and function.__name__
+        thread = TimedThread(seconds, name=name, target=_timed_force_state)
+        self.threads.append(thread)
+        thread.start()
+
+    def stop_timed_function(self, state=None, function=None):
+        """Stop a running thread, while cleaning dead thread"""
+        name = state and (isinstance(state, str) and state or state.name)
+        name = name or function and function.__name__
+        for i, thread in enumerate(self.threads):
+            if thread.name == name or thread.is_dead:
+                thread.stop()
+                self.threads.pop(i)
+
+    def send_autodestroy_message(self, msg, seconds, **kwargs):
+        """Send a message (outside the conversation) that will autodestroy itself"""
+
+        def send_message(tdata):
+            chat_id = tdata.update.effective_chat.id
+            msg_id = tdata.context.bot.send_message(chat_id, msg, **kwargs)
+            sleep(seconds)
+            msg_id.delete()
+
+        self.set_timed_function(0, function=send_message)
+
+    def get_threads(self):
+        """Get running threads, true=alive false=dead"""
+        return {
+            thread.name: thread.is_dead and "dead" or "running"
+            for thread in self.threads
+        }
+
+    # ---- forcing
     def force_state(self, state, update):
         """Force a state in the conversation"""
         if isinstance(state, str):
@@ -299,6 +367,7 @@ class AutoConvHandler:
             self.tData.update_telegram_data(update, context)
         return self.force_state(self.conversation.start, update)
 
+    # ---- main
     def manage_conversation(self, update, context, delete_first=False):
         """Master function for converastion"""
         state = None
@@ -341,7 +410,7 @@ class AutoConvHandler:
                     or (not state.handler and self.tData.update.message.text)
                 )
                 if state.regex and not match(state.regex, str(data)) or not data:
-                    return self._wrong_message(state.regex)
+                    return self._wrong_message(state.regex, data)
                 to_reply = self.tData.context.user_data.get("bot-msg").edit_text
                 self.tData.update.message.delete()
             # ---- next stage
